/**
* CEnemy class, enemy implementation
* @author Created and maintained by Roman Podobedov
* @version 1.0
*/


public class CEnemy extends CHuman
{
	protected int step_direction;
	protected int prev_step_direction;
	protected boolean first_step;
	public int total_in_pits;
	public int in_pit_delay;

	/**
	* Enemy constructor
	*/
	public CEnemy()
	{
		super();
		step_direction = 1;
		prev_step_direction = 1;
		first_step = true;
		man_type = 1;
		total_in_pits = 0;
		in_pit_delay = 80;
	}
	
	/**
	* Calculate length of line
	* @param x1 first point x coordinate
	* @param y1 first point y coordinate
	* @param x2 second point x coordinate
	* @param y2 second point y coordinate
	* @return length of line
	*/
	float CalculateLineLength(int x1, int y1, int x2, int y2)
	{
		return (float)Math.sqrt((float)(x2-x1)*(x2-x1) + (float)(y2-y1)*(y2-y1));	
	}

	/**
	* Do one automatical step (Enemy Artificial Intelligence)
	*/
	public void AutomaticalStep(CLevel level, int man_pos_x, int man_pos_y)
	{
		float l = 100000;
		float l1;
		boolean flag;
		int step_direction_new;

		RecalculateData();
		if (first_step)
		{
			if ((m_pos_x_cell < man_pos_x) && (canRight(level))) step_direction = 2;
			else if ((m_pos_x_cell > man_pos_x) && (canLeft(level))) step_direction = 1;
			else if ((m_pos_y_cell < man_pos_y) && (canDown(level))) step_direction = 4;
			else if ((m_pos_y_cell > man_pos_y) && (canUp(level))) step_direction = 3;

			first_step = false;
		}

		// Enemy is in Pit???
		if (level.ObjectInPit(m_pos_x_cell, m_pos_y_cell))
		{
			in_pit_delay--;
			if (in_pit_delay == 0)
			{
				in_pit = false;
				int tempx = m_pos_x_cell;
				int tempy = m_pos_y_cell;
				m_pos_x = m_pos_x_cell*16;
				m_pos_y = (m_pos_y_cell-1)*16;
				RecalculateData();
				if (step_direction == 1) StepLeft(level);
				if (step_direction == 2) StepRight(level);
				if (step_direction == 3) StepUp(level);
				if (step_direction == 4) StepDown(level);
				level.SetXY(tempx, tempy, (byte)0);		
				in_pit_delay = 80;
				return;
			}
			if (!in_pit) total_in_pits++; // For scoring
			in_pit = true;
			in_the_fly = false;
			level.SetXY(m_pos_x_cell, m_pos_y_cell, (byte)3);
			if (number_of_boxes == 1)
			{
				level.SetXY(m_pos_x_cell, m_pos_y_cell-1, (byte)5);
				number_of_boxes = 0;
			}
			return;
		}		

		// Calculate next position of enemy
		if (((m_pos_x/16)*16 == m_pos_x) && ((m_pos_y/16)*16 == m_pos_y) && (in_the_fly == false)) 
		{
	
			// Can enemy place box here???
			if ((level.GetXY(m_pos_x_cell, m_pos_y_cell) == 0) && (level.GetXY(m_pos_x_cell, m_pos_y_cell-1) != 0) &&
				 (in_the_fly == false) && (number_of_boxes == 1))
			{
				if ((int)(Math.random()*2) == 0)
				{
					number_of_boxes = 0;
					level.SetXY(m_pos_x_cell, m_pos_y_cell, (byte)5);
				}
			}

			flag = false;
			step_direction_new = step_direction;

			if (canUp(level) && (step_direction != 4))
			{				
				flag = true;
				l1 = CalculateLineLength(m_pos_x_cell, m_pos_y_cell-1, man_pos_x, man_pos_y);
				if ((man_pos_y < m_pos_y_cell) && ((int)(Math.random()*3) == 0)) l1 = 0;
				if (l1 < l)	
				{	
					step_direction_new = 3;
					l = l1;
				}
			}
			if (canDown(level) && (step_direction != 3))
			{				
				flag = true;
				l1 = CalculateLineLength(m_pos_x_cell, m_pos_y_cell+1, man_pos_x, man_pos_y);
				if ((man_pos_y > m_pos_y_cell) && ((int)(Math.random()*3) == 0)) l1 = 0;
				if (l1 < l)	
				{
					step_direction_new = 4;
					l = l1;
				}
			}
			if (canRight(level) && (step_direction != 1))
			{				
				flag = true;
				l1 = CalculateLineLength(m_pos_x_cell+1, m_pos_y_cell, man_pos_x, man_pos_y);
				if (l1 < l)
				{	
					step_direction_new = 2;
					l = l1;
				}
			}
			if (canLeft(level) && (step_direction != 2))
			{				
				flag = true;
				l1 = CalculateLineLength(m_pos_x_cell-1, m_pos_y_cell, man_pos_x, man_pos_y);
				if (l1 < l)	
				{
					step_direction_new = 1;
					l = l1;
				}
			}

			// If we can't move (in corner for example)
			if (flag == false)
			{
				// Change direction to negative direction
				if (step_direction == 1) step_direction_new = 2;
				else
					if (step_direction == 2) step_direction_new = 1;
					else
						if (step_direction == 3) step_direction_new = 4;
						else
							if (step_direction == 4) step_direction_new = 3;
			}

			step_direction = step_direction_new;
		}
		if ((in_the_fly == false) && (in_pit == false))
		{
			if (step_direction == 1) StepLeft(level);
			if (step_direction == 2) StepRight(level);
			if (step_direction == 3) StepUp(level);
			if (step_direction == 4) StepDown(level);
		}
	}


	/**
	* Check that UP move is allowed
	*/
	public boolean canUp(CLevel level)
	{
		if ((m_pos_y_cell-1) < 0) return false;

		if (level.GetXY(m_pos_x_cell, m_pos_y_cell-1) == 4) return true;

		if ((level.GetXY(m_pos_x_cell, m_pos_y_cell-1) == 0) &&
		   (level.GetXY(m_pos_x_cell, m_pos_y_cell) == 4)) return true;
		return false;
	}

	/**
	* Check that DOWN move is allowed
	*/
	public boolean canDown(CLevel level)
	{
		if ((m_pos_y_cell+1) >= level.level_height) return false;

		if ((level.GetXY(m_pos_x_cell, m_pos_y_cell+1) == 0) ||
			(level.GetXY(m_pos_x_cell, m_pos_y_cell+1) == 4) ||
			(level.GetXY(m_pos_x_cell, m_pos_y_cell+1) == 5)) return true;
		
		return false;
	}

	/**
	* Check that LEFT move is allowed
	*/
	public boolean canLeft(CLevel level)
	{
		if ((m_pos_x_cell-1) < 0) return false;

		if ((level.GetXY(m_pos_x_cell-1, m_pos_y_cell) == 0) || 
			(level.GetXY(m_pos_x_cell-1, m_pos_y_cell) == 4) ||
			(level.GetXY(m_pos_x_cell-1, m_pos_y_cell) == 5) ||
			(level.GetXY(m_pos_x_cell-1, m_pos_y_cell) == 6)) return true;
		return false;
	}

	/**
	* Check that RIGHT move is allowed
	*/
	public boolean canRight(CLevel level)
	{
		if ((m_pos_x_cell+1) >= level.level_width) return false;

		if ((level.GetXY(m_pos_x_cell+1, m_pos_y_cell) == 0) || 
			(level.GetXY(m_pos_x_cell+1, m_pos_y_cell) == 4) ||
			(level.GetXY(m_pos_x_cell+1, m_pos_y_cell) == 5) ||
			(level.GetXY(m_pos_x_cell+1, m_pos_y_cell) == 6)) return true;
		return false;
	}
}
