/**
* CEngine class, main engine of game
* @author Created and maintained by Roman Podobedov
* @version 1.0
*/

import java.net.*;

public class CEngine
{
	public CMan man; // Our hero
	public CEnemy enemies[]; // Level enemies
	public CLevel level; // Our level
	public int cell_width, cell_height;
	protected int tick;

	public int lives; // Total lives
	public int score; // Game score
		
	public boolean hero_is_dead;
	public boolean level_is_complete;

	public boolean started; // True - if engine started

	/**
	* Constructor of engine
	*/
	public CEngine()
	{
		man = new CMan();
		level = new CLevel();
		Init();
	}
	
	/*
	* Engine initialization
	*/
	public void Init()
	{
		tick = 0;
		score = 0;
		lives = 10;
		hero_is_dead = false;
		level_is_complete = false;
		started = false;
	}
	
	/**
	* Set all cell parameters in engine
	* @param width Width of one cell
	* @param height Height of one cell
	*/
	public void SetCellParam(int width, int height)
	{
		cell_width = width;
		cell_height = height;
		level.cell_width = width;
		level.cell_height = height;
		man.cell_width = width;
		man.cell_height = height;
	}

	/**
	* Start engine
	* @param level_num URL of level
	*/
	public void Engine_Start(URL level_num) throws Exception
	{
		level = new CLevel();

		hero_is_dead = false;
		level.pits = null;
		level_is_complete = false;
		man.number_of_boxes = 0;
		man.number_of_boxes_score = 0;
	
		// Load level from file
		level.LoadFromFile(level_num);
		
		man.is_dead = false;
		enemies = new CEnemy[4];
		for (int i=0; i<level.total_enemy; i++)
		{
			enemies[i] = new CEnemy();
			enemies[i].SetPosition(level.enemy_coords[i][0]*cell_width, level.enemy_coords[i][1]*cell_height);
			enemies[i].cell_width = cell_width;
			enemies[i].cell_height = cell_height;
		}
		man.SetPosition(level.GetManX()*cell_width, level.GetManY()*cell_height);
		started = false;
	}

	/**
	* Do step left
	*/
	public void StepLeft()
	{
		started = true;
		man.StepLeft(level);
	}

	/**
	* Do step right
	*/
	public void StepRight()
	{
		started = true;
		man.StepRight(level);
	}

	/**
	* Do step up
	*/
	public void StepUp()
	{
		started = true;
		man.StepUp(level);
	}

	/**
	* Do step down
	*/
	public void StepDown()
	{
		started = true;
		man.StepDown(level);
	}

	/**
	* Do hit
	*/
	public void Hit()
	{
		started = true;
		man.Hit(level);
	}
	
	/**
	* Do some calculations in level
	* @return true - if hero can't move (in the fly), false - otherwise
	*/
	public boolean DoCalculations()
	{
		boolean temp;

		if (!started) return false;

		// Calculate score
		score += man.number_of_boxes_score*250;
		man.number_of_boxes_score = 0;
		
		for (int i=0; i<level.total_enemy; i++)
		{
			score += enemies[i].total_in_pits*125;
			enemies[i].total_in_pits = 0;
		}	

		level.DoPitTick();
	
		man.RecalculateData();
		tick++;
		if (tick == 2)
		{
			for (int i=0; i<level.total_enemy; i++)
			{
				enemies[i].AutomaticalStep(level, man.m_pos_x_cell, man.m_pos_y_cell);
				enemies[i].DoCalculations(level);
				enemies[i].RecalculateData();
				if ((enemies[i].m_pos_x_cell == man.m_pos_x_cell) &&
				   (enemies[i].m_pos_y_cell == man.m_pos_y_cell)) man.is_dead = true;
			}	
			tick = 0;
		}
		temp = man.DoCalculations(level);

		if (man.is_dead) hero_is_dead = true;
		if (man.CheckLevelIsComplete(level))
		{
			// Level is complete!
			level_is_complete = true;
		}
		
		return temp;
	}
}
