/**
* CHuman class, abstract human implementation
* @author Created and maintained by Roman Podobedov
* @version 1.0
*/

public class CHuman
{
	protected int max_sprites = 3; // Maximum count of sprites
	protected int m_pos_x, m_pos_y; // Human position in pixels
	public int m_pos_x_cell, m_pos_y_cell; // Position in cells
	protected boolean m_only_one_cell; // True - when man use only one cell
	protected boolean m_only_one_cell_updown; // True - when man use only one cell (can up, down)
	protected boolean m_only_one_cell_leftright; // True - when man use only one cell (can left, right)
	protected int direction; // Human direction (0 - Left, 1 - Right, 2 - Up , 3 - Down, 4 - center)
	public boolean onRope; // True when human is on rope
	protected int sprite_num; // Current sprite number
	public int cell_width, cell_height; // Cell width and height
	protected int number_of_boxes; // Total boxes, that have human
	public int number_of_boxes_score;
	protected boolean in_the_fly;
	protected int man_type; // 0 - Hero, 1 - Enemy
	public boolean is_dead;
	public boolean in_pit;

	/**
	* Constructor
	*/
	public CHuman()
	{
		sprite_num = 0;
		direction = 4;
		m_only_one_cell = true;
		m_only_one_cell_updown = true;
		m_only_one_cell_leftright = true;
		onRope = false;
		number_of_boxes = 0;
		in_the_fly = false;
		man_type = 0;
		is_dead = false;
		in_pit = false;
	}

	/**
	* Set human position (pixels)
	* @param x X Position on human (in pixels)
	* @param y Y Position on human (in pixels)
	*/
	public void SetPosition(int x, int y)
	{
		m_pos_x = x;
		m_pos_y = y;
	}

	/**
	* Get direction of human
	* @return Current direction of human
	*/
	public int GetDirection()
	{
		return direction;
	}

	/**
	* Recalculates all positioning data in class (must be called, when any change is maded)
	*/
	public void RecalculateData()
	{
		// Calculate position in cells
		m_pos_x_cell = m_pos_x / cell_width;
		m_pos_y_cell = m_pos_y / cell_height;
		
		// This position takes only one cell (up, down)?
		if (m_pos_x_cell*cell_width == m_pos_x) m_only_one_cell_updown = true;
		else m_only_one_cell_updown = false;

		// This position takes only one cell (left, right)?
		if (m_pos_y_cell*cell_height == m_pos_y) m_only_one_cell_leftright = true;
		else m_only_one_cell_leftright = false;

		if ((m_only_one_cell_updown) && (m_only_one_cell_leftright)) m_only_one_cell = true;
	}

	/**
	* Get current sprite number
	* @return Sprite number
	*/
	public int GetSpriteNum()
	{
		return sprite_num;
	}

	/**
	* Get current X position
	* @return X position (in pixels)
	*/
	public int GetPosX()
	{
		return m_pos_x;
	}

	/**
	* Get current Y position
	* @return Y position (in pixels)
	*/
	public int GetPosY()
	{
		return m_pos_y;
	}
	
	/**
	* Do one step right
	* @param level pointer to level
	*/
	public boolean StepRight(CLevel level)
	{
		// Calculate current position
		RecalculateData();
		byte block_type_first = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// Calculate right position
		m_pos_x += cell_width;
		RecalculateData();
		byte block_type = level.GetXY(m_pos_x_cell, m_pos_y_cell);
		
		// If step allowed
		if (((block_type == 0) || (block_type > 3)) && (m_only_one_cell_leftright))
		{
			m_pos_x -= cell_width;
			m_pos_x += cell_width/4;
			CheckBounds(level);
			IncreaseSprite();

			direction = 1;
			return true;
		}
		// If not allowed
		m_pos_x -= cell_width;
		RecalculateData();
		return false;
	}

	/**
	* Do one step left
	* @param level pointer to level
	*/
	public boolean StepLeft(CLevel level)
	{
		// Calculate current position
		RecalculateData();
		byte block_type_first = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// Calculate left position
		m_pos_x -= cell_width/4;
		RecalculateData();		
		byte block_type = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// If step allowed
		if (((block_type == 0) || (block_type > 3)) && (m_only_one_cell_leftright))
		{
			CheckBounds(level);
			IncreaseSprite();
			direction = 0;
			return true;
		}
		m_pos_x += cell_width/4;
		RecalculateData();
		return false;
	}

	/**
	* Do one step up
	* @param level pointer to level
	*/
	public boolean StepUp(CLevel level)
	{
		// Do step up
		m_pos_y--;
		RecalculateData();
		byte block_type_up = level.GetXY(m_pos_x_cell, m_pos_y_cell);
		m_pos_y++;

		m_pos_y += (cell_height-1);
		RecalculateData();

		byte block_type = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// If step allowed
		if ((block_type == 4) && (m_only_one_cell_updown) &&
		 (block_type_up != 1) && (block_type_up != 2) && (block_type_up != 3))
		{
			m_pos_y -= (cell_height-1);
			m_pos_y -= cell_height/4;
			CheckBounds(level);
			IncreaseSprite();
			direction = 2;
			return true;
		}
		m_pos_y -= (cell_height-1);
		RecalculateData();
		return false;
	}

	/**
	* Do one step down
	* @param level pointer to level
	*/
	public boolean StepDown(CLevel level)
	{
		// Calculate current position
		RecalculateData();
		byte block_type_first = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// Calculate right down position
		m_pos_x += cell_width-1;
		m_pos_y += cell_height;
		RecalculateData();
		byte block_type_right_down = level.GetXY(m_pos_x_cell, m_pos_y_cell);
		// Restore previous position
		m_pos_x -= cell_width-1;
		m_pos_y -= cell_height;
		RecalculateData();

		m_pos_y += cell_height;
		RecalculateData();
		byte block_type = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		if ((block_type_first == 6) && (block_type_right_down == 0) && (block_type == 0))
		{
			m_pos_y -= cell_height;
			m_pos_y += cell_height/4;
			CheckBounds(level);
			IncreaseSprite();
			direction = 3;
			return true;
		} 
		if ((block_type == 4) && (m_only_one_cell_updown))
		{
			m_pos_y -= cell_height;
			m_pos_y += cell_height/4;
			CheckBounds(level);
			IncreaseSprite();
			direction = 3;
			return true;
		}
		if (!((block_type == 1) || (block_type == 3)) && (m_only_one_cell_updown))
		{
			m_pos_y -= cell_height;
			m_pos_y += cell_height/4;
			CheckBounds(level);
			IncreaseSprite();
			direction = 3;
			return true;
		}
		m_pos_y -= cell_height;
		RecalculateData();
		return false;
	}

	/**
	* Do one hit
	* @param level pointer to level
	*/
	public void Hit(CLevel level)
	{
		// Save this position
		int save_x = m_pos_x;
		int save_y = m_pos_y;

		RecalculateData();

		if (direction == 0)
		{
			if (m_pos_x != (m_pos_x_cell * cell_width))
				m_pos_x = (m_pos_x_cell+1) * cell_width;

			CheckBounds(level);
			RecalculateData();
			byte block_type_down_left = level.GetXY(m_pos_x_cell-1, m_pos_y_cell+1);
			byte block_type_left = level.GetXY(m_pos_x_cell-1, m_pos_y_cell);
			if ((block_type_down_left == 1) && ((block_type_left == 0) || (block_type_left == 6) || (block_type_left == 7)))
			{
				level.AddNewPit(m_pos_x_cell-1, m_pos_y_cell+1);
			}
			else
			{
				m_pos_x = save_x;
				m_pos_y = save_y;
				RecalculateData();
				return;
			}
		}
		else
		if (direction == 1)
		{
			if (m_pos_x != (m_pos_x_cell * cell_width))
				m_pos_x = (m_pos_x_cell) * cell_width;

			CheckBounds(level);
			RecalculateData();
			byte block_type_down_right = level.GetXY(m_pos_x_cell+1, m_pos_y_cell+1);
			byte block_type_right = level.GetXY(m_pos_x_cell+1, m_pos_y_cell);
			if ((block_type_down_right == 1) && ((block_type_right == 0) || (block_type_right == 6) || (block_type_right == 7)))
			{
				level.AddNewPit(m_pos_x_cell+1, m_pos_y_cell+1);
			}
			else
			{
				m_pos_x = save_x;
				m_pos_y = save_y;
				RecalculateData();
				return;
			}
		}
	}

	/**
	* Increase current sprite of human
	*/
	protected void IncreaseSprite()
	{
		sprite_num++;
		if (sprite_num >= max_sprites) sprite_num = 0;
	}

	/**
	* Check current human position in level
	* @param level pointer to level
	*/
	public void CheckBounds(CLevel level)
	{
		if (m_pos_x < 0) m_pos_x = 0;
		if (m_pos_y < 0) m_pos_y = 0;
		if (m_pos_x >= cell_width*(level.level_width-1)) m_pos_x = cell_width*(level.level_width-1);
		if (m_pos_y >= cell_height*(level.level_height-1)) m_pos_y = cell_height*(level.level_height-1);
	}

	/**
	* Do some calculations for human
	* @param level pointer to level
	* @return True - if keyboard must be blocked (human on the fly), False - otherwise
	*/
	public boolean DoCalculations(CLevel level)
	{
		int positions[][];
		int deploy_pos, i, j;
		byte pos_data;

		in_the_fly = false;
		RecalculateData();
		byte block_type_this = level.GetXY(m_pos_x_cell, m_pos_y_cell);
		
		if ((is_dead == true) && (man_type == 1))
		{
			// Search new position for enemy
			positions = new int[32][2];
			for (i=0; i<22; i++)
			{	
				deploy_pos = 0;
				for (j=0; j<32; j++)
				{
					pos_data = level.GetXY(j, i);
					if ((pos_data == 0) || (pos_data == 4) || (pos_data == 5) ||
					   (pos_data == 6) || (pos_data == 7))
					   {
							// Can deploy here
							positions[deploy_pos][0] = j; // x position
							positions[deploy_pos][1] = i; // y position
							deploy_pos++;
					   }
				}
				if (deploy_pos > 0)
				{
					int rand_el = (int)(Math.random()*deploy_pos);
					m_pos_x = positions[rand_el][0]*16;
					m_pos_y = positions[rand_el][1]*16;
					break;
				}
			}

			is_dead = false;
			in_pit = false;
			RecalculateData();
			return false;
		}

		if (block_type_this == 1)
		{
			is_dead = true;
			return false;
		}

		if (in_pit)
		{
			return false;
		}
		// Check, that man takes one box
		if ((block_type_this == 5) && (m_only_one_cell_updown)) 
		{
			if (man_type == 0)
			{
				number_of_boxes++;
				number_of_boxes_score++;
				level.SetXY(m_pos_x_cell, m_pos_y_cell, (byte)0);
				if (number_of_boxes == level.GetBoxNumber()) level.Finishing();
			}
			else
			{
				if ((number_of_boxes == 0) && ((int)(Math.random()*5) == 0))
				{
					number_of_boxes++;
					level.SetXY(m_pos_x_cell, m_pos_y_cell, (byte)0);
				}
			}
		}
		
		// Check that man is on rope
		if ((block_type_this == 6) && (m_only_one_cell_leftright)) 
		{
			onRope = true;
			return false;
		}
		onRope = false;

		m_pos_x += cell_width-1;
		m_pos_y += cell_height;
		RecalculateData();
		byte block_type_right_down = level.GetXY(m_pos_x_cell, m_pos_y_cell);
		m_pos_x -= cell_width-1;
		m_pos_y -= cell_height;
		RecalculateData();

		if (m_pos_y_cell == level.level_height-1) return false;

		m_pos_y += cell_height;
		RecalculateData();
		byte block_type = level.GetXY(m_pos_x_cell, m_pos_y_cell);

		// Check that man can fly
		if (((block_type == 0) || (block_type == 2) || (block_type == 5) || (block_type == 6) || (block_type == 7)) &&
			((block_type_this != 4)) &&
			((block_type_right_down == 0) || (block_type_right_down == 2) || (block_type_right_down == 5) || (block_type_right_down == 6) || (block_type_right_down == 7)))
		{
			m_pos_y -= cell_height;
			m_pos_y += cell_height/4;
			CheckBounds(level);
			direction = 4;
			in_the_fly = true;
			return true; // In the fly
		}
		m_pos_y -= cell_height;
		RecalculateData();
		return false;
	}
}
