/**
* CLevel class, contains all information about level
* @author Created and maintained by Roman Podobedov
* @author Programming: Roman Podobedov, Vitaly Petrov
* @version 1.0
*/

import java.net.*;
import java.io.*;

public class CLevel
{
	public byte level_width = 32;
	public byte level_height = 22;
	public int cell_width, cell_height;

	protected byte level_data[][]; // Level data (all info about level objects)
	protected int man_x, man_y; // Coordinates (in blocks) of hero start position
	public int total_enemy; // Total enemies N in level
	public int enemy_coords[][]; // Array of coordinates of N enemies: enemy_coords[N][2];
	protected int gold_box;//Total number of gold boxes  

	CPit pits; // All pits on the level (dynamic queue)

	public boolean on_finishing_stage;
			
	/**
	* Main constructor
	*/
	public CLevel()
	{
		// Level initialization
		level_data = new byte[level_height][level_width];
		gold_box = 0;
		pits = null;
		on_finishing_stage = false;
	}

	/**
	* Load all level data from file
	* @param level_name URL of level
	*/
	public void LoadFromFile(URL level_name) throws Exception
	{
		String line;
		int num = 1;
		on_finishing_stage = false;
		
		URLConnection LevUrlConnection = level_name.openConnection();

		try
		{
			InputStream input = LevUrlConnection.getInputStream();
	  		BufferedReader text_in = new BufferedReader(new InputStreamReader(input));

	  		line = text_in.readLine();
	  		while (line != null)
			{
				if (num == 1)
				{		
					if (line.charAt(1)==' ') man_x = Integer.parseInt(line.substring(0,1));					
					else man_x = Integer.parseInt(line.substring(0,2));
					if (man_x > 9) man_y = Integer.parseInt(line.substring(3));
					if (man_x < 10) man_y = Integer.parseInt(line.substring(2));
				}
				
				if (num == 2) 
				{
					total_enemy = Integer.parseInt(line);
					enemy_coords = new int[total_enemy][2];
				}
				
				if ((num > 2) && (num < total_enemy + 3))
				{
					if (line.charAt(1)==' ') enemy_coords[num-3][0] = Integer.parseInt(line.substring(0,1));
					else enemy_coords[num-3][0] = Integer.parseInt(line.substring(0,2));
					if (enemy_coords[num-3][0] > 9) enemy_coords[num-3][1] = Integer.parseInt(line.substring(3));
					if (enemy_coords[num-3][0] < 10) enemy_coords[num-3][1] = Integer.parseInt(line.substring(2));
				}
				
				if (num > total_enemy + 2)
				{
					for (int t = 0; t < level_width; t++)
					{
						SetXY(t, num-total_enemy-3, Byte.parseByte(line.substring(t, t+1)));
						SetBoxNumber(Byte.parseByte(line.substring(t, t+1)));
					}
				}

				line = text_in.readLine();
				num++;
			}
			
			text_in.close();
			input.close();

		}
		catch (IOException e)
		{
			System.out.println("Error: " + e);
		}
	}
	
	/**
	* Get element at x, y position (position as block)
	* @param x X coordinate (in cells)
	* @param y Y coordinate (in cells)
	* @return Cell data
	*/
	public byte GetXY(int x, int y)
	{
		if ((x >= level_width) || (x < 0)) return 0;
		if ((y >= level_height) || (y < 0)) return 0;
		return level_data[y][x];
	}

	/**
	* Set element at x, y position
	* @param x X coordinate (in cells)
	* @param y Y coordinate (in cells)
	* @param element Cell data
	*/
	public void SetXY(int x, int y, byte element)
	{
		if ((x >= level_width) || (x < 0) || (y >= level_height) || (y < 0)) return;
		level_data[y][x] = element;
	}


	/**
	* Get x hero position
	* @return hero x position (in pixels)
	*/
	public int GetManX() { return man_x; }

	/**
	* Get y hero position
	* @return hero y position (in pixels)
	*/
	public int GetManY() { return man_y; }

	/**
	* Add new pit in level
	* @param x x position (in cells)
	* @param y y position (in cells)
	*/
	public void AddNewPit(int x, int y)
	{
		SetXY(x, y, (byte)0);
		
		// Add new pit to queue
		CPit temp_pit = new CPit();
		temp_pit.next = pits;
		pits = temp_pit;
		pits.x = x;
		pits.y = y;
	}

	/**
	* Check that object in the pit
	* @param x object x position (in cells)
	* @param y object y position (in cells)
	* @return True - if object in the Pit, False - otherwise
	*/	
	public boolean ObjectInPit(int x, int y)
	{
		CPit temp_pit;

		temp_pit = pits;

		while (temp_pit != null)
		{
			if ((temp_pit.time_to_step != 0) && (temp_pit.x == x) && (temp_pit.y == y)) return true;
			temp_pit = temp_pit.next;
		}
		return false;
	}

	/**
	* Do for all pits one tick
	*/
	public void DoPitTick()
	{
		CPit temp_pit;
		CPit prev_pit;

		temp_pit = pits;
		prev_pit = null;

		while (temp_pit != null)
		{
			temp_pit.DoStep(this);
			if (temp_pit.is_dead)
			{
				// Delete current pit
				if (prev_pit != null) prev_pit.next = temp_pit.next;
				else
				{
					pits = temp_pit.next;
					temp_pit = pits;
					continue;
				}
			}
			prev_pit = temp_pit;
			temp_pit = temp_pit.next;
		}
	}

 	/**
	* Set number of gold boxes
	*/
	void SetBoxNumber(byte n)
	{
		if (n==5) gold_box++;	
	}

	/**
	* Get number of gold boxes
	*/
	public int GetBoxNumber()
	{
		return gold_box;
	}

	/**
	* If all boxes are collected then replace invisible elements to visible elements
	*/
	public void Finishing()
	{
		for (int i=0; i<level_height; i++)
			for (int j=0; j<level_width; j++)
				if (level_data[i][j] == 7) level_data[i][j] = 4;

		on_finishing_stage = true;
	}
}

